
rm(list = ls()); gc()
setwd("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/")
recipe_file <- "recipes/recipe_1_nadia_scorecards.yml"

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")

for (smonth in 1:12) {
## Define recipe needed to load calibrated data and skill metrics
  recipe <- read_yaml(recipe_file)
  recipe$Analysis$Time$sdate <- paste0(sprintf("%02d", smonth), '01')
## Load datasets
  data <- load_datasets(recipe_file)
## Compute skill metrics of raw data
  skill_metrics <- compute_skill_metrics(data$hcst, data$obs, recipe) 
  archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive.yml"))$archive
  save_data(recipe, archive, data, skill_metrics = skill_metrics)  
}


##### Script requires module Phantomjs, currently only available in Nord3 
##### Load Phantomjs in Nord3 before running script: module load Phantomjs

##### To open R in Nord·: 
#  module load R/4.1.2-foss-2019b
#  module load OpenMPI/4.0.5-GCC-8.3.0-nord3-v2
#  R


### Temporarily to be able to run without loading data first ###
setwd("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/")
library(yaml)
recipe_file <- "recipes/recipe_1_nadia_scorecards.yml"
recipe <- read_yaml(recipe_file)
archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive.yml"))$archive
variable_dictionary <- read_yaml(paste0(recipe$Run$code_dir, "conf/variable-dictionary.yml"))$vars
type_dictionary <- read_yaml(paste0(recipe$Run$code_dir, "conf/scorecard_type.yml"))$types

input.path <- '/esarchive/scratch/nmilders/scorecards_data/input_data'
fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results'

currentworkingdir <- getwd()

source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_0_configuration_functions.R")

## Load data files
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_1_load_metrics.R")
loaded_metrics <- SC_load_metrics(recipe)

###### Define scorecard type and region #######
scorecard_type <- "type-9" 
region <- 1 ## only for multiple system scorecards (type 9 - 12)

## Transform metrics 
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_2_transform_loaded_metrics.R")
transformed_metrics <- transform_loaded_metrics(recipe, loaded_metrics)

## Create scorecard
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_3_create_scorecard.R")
scorecard_table <- SC_create_table(transformed_metrics)

## Save scorecard
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_4_save_scorecard.R")
save_scorecard <- SC_save_image(system.filename, reference.filename, var.name, var.filename, period, scorecard_type)

setwd(currentworkingdir)


#################### PARAMETERS ########################

# start.year <- recipe$Analysis$Time$hcst_start ## 1993
# end.year <- recipe$Analysis$Time$hcst_end ## 2016
# 
# system.filename <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5'
# system.name <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_name ## 'ECMWF System 5
# 
# reference.name <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_name ## 'ERA5'
# reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
# 
# var.filename <- recipe$Analysis$Variables$name ## 'tas'
# var.name <- variable-dictionary$vars[[recipe$Analysis$Variables$name]]$long_name ##
# var.unit <- variable-dictionary$vars[[recipe$Analysis$Variables$name]]$units ## 'K'

#plevel <- '' # because single levels

