################################################################################
######### Script to load scorecard metrics data for visualization ##############
################################################################################

transform_loaded_metrics <- function(recipe, loaded_metrics){
    
    plevel <- "" # TO DO: read from recipe
    start.year <- recipe$Analysis$Time$hcst_start ## 1993
    end.year <- recipe$Analysis$Time$hcst_end ## 2016
    var.name <- variable_dictionary[[recipe$Analysis$Variables$name]]$long_name
    system.filename <- list('ecmwfs5','dwds21') ## archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5'
    reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
    var.filename <- recipe$Analysis$Variables$name ## 'tas'
    period <- paste0(start.year, "-", end.year)
    
    n.systems <- length(list('ecmwfs5', 'dwds21'))
    lead.months <- indices(recipe$Analysis$Time$ftime_min : recipe$Analysis$Time$ftime_max)
    
    ## TO DO:
    ## Need to loop over system and start date (?)
    ## for system intercomparison scorecard, extra loop need for region
    
    ## Define regions ## TO DO: make region flexible
    region.names <- initialfunction_regionnames_and_borders()[,1]
    region.names.figure <- initialfunction_regionnames_and_borders()[,2]
    lat.min <- initialfunction_regionnames_and_borders()[,3]
    lat.max <- initialfunction_regionnames_and_borders()[,4]
    lon.min <- c() #initialfunction_regionnames_and_borders()[,5]
    lon.max <- c() #initialfunction_regionnames_and_borders()[,6]
    n.regions <- length(region.names)
    
    ## Load metrics from the 12 netCDF files of the chosen forecast system, one for each start month:
    start.months <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12") ## TO DO: use month.abb functionrecipe
    n.start.months <- length(start.months)
    
    ## Forecast months to extract
    my.ftime.months <- 1:6 ## select the first 6 forecast months 
    n.ftime.months <- length(my.ftime.months)
    
    ## number of regions
    n.regions <- length(region.names) ## for system intercomparison
    
    ### Define metrics for single level variable scorecard
    if(scorecard_type == 'type-1' || scorecard_type == 'type-2' || scorecard_type == 'type-3' ||  scorecard_type == 'type-4'){
      
      bias <- corr <- rpss <- crpss <- enssprerr <- list()
      
        for(m in start.months){
          mm <- as.numeric(m)

          ## Define metrics
          bias_values <- loaded_metrics[[system.filename]][[mm]]$mean_bias 
          corr_values <- loaded_metrics[[system.filename]][[mm]]$enscorr
          rpss_values <- loaded_metrics[[system.filename]][[mm]]$rpss
          crpss_values <- loaded_metrics[[system.filename]][[mm]]$crpss
          enssprerr_values <-loaded_metrics[[system.filename]][[mm]]$enssprerr
          
          bias <- append(bias, list(bias_values))
          corr <- append(corr, list(corr_values))
          rpss <- append(rpss, list(rpss_values))
          crpss <- append(crpss, list(crpss_values))
          enssprerr <- append(enssprerr, list(enssprerr_values))
        } ## close if on start.months
        
        ## Load lat and lon values
        lat.netCDF <- attributes(loaded_metrics[[system.filename]])$lat ## $vals ??
        lon.netCDF <- attributes(loaded_metrics[[system.filename]])$lon ## $vals ??

      ## position of lat and lon min and max values inside the netCDF
      pos.lat.min <- pos.lat.max <- pos.lon.min <- pos.lon.max <- c()
      for(i in 1:n.regions){
        if(is.null(lat.min)) pos.lat.min[i] <- which(lat.netCDF == min(lat.netCDF)) else pos.lat.min[i] <- initialfunction_position_latlon(as.integer(lat.min[i]), lat.netCDF)
        if(is.null(lat.max)) pos.lat.max[i] <- which(lat.netCDF == max(lat.netCDF)) else pos.lat.max[i] <- initialfunction_position_latlon(as.integer(lat.max[i]), lat.netCDF)
        if(is.null(lon.min)) pos.lon.min[i] <- which(lon.netCDF == min(lon.netCDF)) else pos.lon.min[i] <- initialfunction_position_latlon(as.integer(lon.min[i]), lon.netCDF)
        if(is.null(lon.max)) pos.lon.max[i] <- which(lon.netCDF == max(lon.netCDF)) else pos.lon.max[i] <- initialfunction_position_latlon(as.integer(lon.max[i]), lon.netCDF)
      }
    
      ## Empty arrays with the score values to plot:
      mean.bias <- mean.corr <- mean.crpss <- mean.rpss <- mean.enssprerr <- array(NA, c(n.start.months, n.regions, n.ftime.months))

      for(i in 1:n.regions){
        for(m in start.months){
          mm <- as.numeric(m)
          
          ## Select only scores inside the target region: [lon, lat, n.ftime.months]
          bias.region <- bias[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
          corr.region <- corr[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
          crpss.region <- crpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
          rpss.region <- rpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
          enssprerr.region <- enssprerr[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
          
          ## Replace values with (-)Inf with NA or the average will be (-)Inf too: (but infinite values should be absent in theory)
          rpss.region[which(is.infinite(rpss.region))] <- NA 
          crpss.region[which(is.infinite(crpss.region))] <- NA 
          enssprerr.region[which(is.infinite(enssprerr.region))] <- NA 
          
          ## Average score values inside the target region: 
          mean.bias[mm, i,] <- apply(bias.region, 3, mean, na.rm = TRUE) ## [n.start.months, n.regions, n.ftime.months]
          mean.corr[mm, i,] <- apply(corr.region, 3, mean, na.rm = TRUE)
          mean.rpss[mm, i,] <- apply(rpss.region, 3, mean, na.rm = TRUE)
          mean.crpss[mm, i,] <- apply(crpss.region, 3, mean, na.rm = TRUE)
          mean.enssprerr[mm, i,] <- apply(enssprerr.region, 3, mean, na.rm = TRUE)
          
        } ## close if on start.months
      } ## close if on n.regions
    } ## close if on scorecard_type    
    
    ### Define metrics for system intercomparison scorecard
    if(scorecard_type == 'type-9' || scorecard_type == 'type-10' || scorecard_type == 'type-11' ||  scorecard_type == 'type-12'){

      #  bias <- corr <- rpss <- crpss <- enssprerr <- list()
      #  mean.bias <- mean.corr <- mean.crpss <- mean.rpss <- mean.enssprerr <- array(NA, c(n.start.months, n.systems, n.ftime.months))
        n.metrics <- 5 # correct!!
        met_by_reg <- array(NA, c(system = n.systems, sdate = n.sdate, 
                                   metric = n.metrics, ftime = n.ftime.months,
                                   region = length(regions)))
        for(sys in 1:n.systems){
          for(m in start.months){
            for(kk in 1:length(n.metrics)) {}
            #mm <- as.numeric(m)
            #pos.mm <- mm + (sys - 1) * n.start.months
              # TODO: do we want only 1 region?
              met_by_reg[sys,m,kk,,1] <- WeightedMean(loaded_metrics[[sys]][[mm]][[kk]],
                                  lon = attributes(loaded_metrics[[sys]])$lon,
                                  lat = attributes(loaded_metrics[[sys]])$lat,
                                  region = regions[[1]])
          }
            ## Define metrics
            bias_values <- loaded_metrics[[sys]][[mm]]$mean_bias 
            corr_values <- loaded_metrics[[sys]][[mm]]$enscorr
            rpss_values <- loaded_metrics[[sys]][[mm]]$rpss
            crpss_values <- loaded_metrics[[sys]][[mm]]$crpss
            enssprerr_values <-loaded_metrics[[sys]][[mm]]$enssprerr
            
            bias <- append(bias, list(bias_values))
            corr <- append(corr, list(corr_values))
            rpss <- append(rpss, list(rpss_values))
            crpss <- append(crpss, list(crpss_values))
            enssprerr <- append(enssprerr, list(enssprerr_values))
          } ## close if on m
          
          ## Load lat and lon values
          lat.netCDF <- attributes(loaded_metrics[[sys]])$lat ## $vals ??
          lon.netCDF <- attributes(loaded_metrics[[sys]])$lon ## $vals ??

          ## position of lat and lon min and max values inside the netCDF:
          pos.lat.min <- pos.lat.max <- pos.lon.min <- pos.lon.max  <- c()
          pos.lon <- pos.lat <- list()
          
          for(i in 1:n.regions){        
            ## pos.lat.min and pos.lat.max:
            if(is.null(lat.min)) pos.lat.min[i] <- which(lat.netCDF == min(lat.netCDF)) else pos.lat.min[i] <- initialfunction_position_latlon(as.integer(lat.min[i]), lat.netCDF)
            if(is.null(lat.max)) pos.lat.max[i] <- which(lat.netCDF == max(lat.netCDF)) else pos.lat.max[i] <- initialfunction_position_latlon(as.integer(lat.max[i]), lat.netCDF)  
            
            ## pos.lon.min and pos.lon.max:
            if(is.null(lon.min)) pos.lon.min[i] <- which(lon.netCDF == min(lon.netCDF)) else pos.lon.min[i] <- initialfunction_position_latlon(as.integer(lon.min[i]), lon.netCDF)
            if(is.null(lon.max)) pos.lon.max[i] <- which(lon.netCDF == max(lon.netCDF)) else pos.lon.max[i] <- initialfunction_position_latlon(as.integer(lon.max[i]), lon.netCDF)
            
            ## pos.lat and pos.lon:
            pos.lat[[i]] <- pos.lat.max[i]:pos.lat.min[i]
            pos.lon[[i]] <- pos.lon.min[i]:pos.lon.max[i]
            
            ## if lon.max > lon.min, pos.lon is made up of two intervals:
            if(!is.null(lon.min[i]) && !is.null(lon.max[i]) && lon.min[i] > lon.max[i]) pos.lon[[i]] <- c(1:pos.lon.max[i], pos.lon.min[i]:n.lon.netCDF)
          }
          
          # pos.lat.min <- pos.lat.max <- pos.lon.min <- pos.lon.max <- c() ## needs to be inside systems loop since different systems can have different grids
          # for(r in region){
          #   if(is.null(lat.min)) pos.lat.min[r] <- which(lat.netCDF == min(lat.netCDF)) else pos.lat.min[r] <- initialfunction_position_latlon(as.integer(lat.min[r]), lat.netCDF)
          #   if(is.null(lat.max)) pos.lat.max[r] <- which(lat.netCDF == max(lat.netCDF)) else pos.lat.max[r] <- initialfunction_position_latlon(as.integer(lat.max[r]), lat.netCDF)
          #   if(is.null(lon.min)) pos.lon.min[r] <- which(lon.netCDF == min(lon.netCDF)) else pos.lon.min[r] <- initialfunction_position_latlon(as.integer(lon.min[r]), lon.netCDF)
          #   if(is.null(lon.max)) pos.lon.max[r] <- which(lon.netCDF == max(lon.netCDF)) else pos.lon.max[r] <- initialfunction_position_latlon(as.integer(lon.max[r]), lon.netCDF)
          # }
         
          for(m in start.months){
            mm <- as.numeric(m)
            pos.mm <- mm + (sys - 1) * n.start.months
            
            ## Select only scores inside the target region: [lon, lat, n.ftime.months]
            bias.region <- bias[[pos.mm]][pos.lon.min:pos.lon.max, pos.lat.max:pos.lat.min, my.ftime.months]
            corr.region <- corr[[pos.mm]][pos.lon.min:pos.lon.max, pos.lat.max:pos.lat.min, my.ftime.months]
            crpss.region <- crpss[[pos.mm]][pos.lon.min:pos.lon.max, pos.lat.max:pos.lat.min, my.ftime.months]
            rpss.region <- rpss[[pos.mm]][pos.lon.min:pos.lon.max, pos.lat.max:pos.lat.min, my.ftime.months]
            enssprerr.region <- enssprerr[[pos.mm]][pos.lon.min:pos.lon.max, pos.lat.max:pos.lat.min, my.ftime.months]
            
            ## Replace values with (-)Inf with NA or the average will be (-)Inf too: (but infinite values should be absent in theory)
            rpss.region[which(is.infinite(rpss.region))] <- NA 
            crpss.region[which(is.infinite(crpss.region))] <- NA 
            enssprerr.region[which(is.infinite(enssprerr.region))] <- NA 
          
            dim_ftime <- which(names(dim(bias.region)) == 'time')
   
            ## Average score values inside the target region: 
            mean.bias[mm,sys,] <- apply(bias.region, dim_ftime, mean, na.rm = TRUE) ## [n.start.months, n.systems, n.ftime.months]
            mean.corr[mm,sys,] <- apply(corr.region, dim_ftime, mean, na.rm = TRUE)
            mean.rpss[mm,sys,] <- apply(rpss.region, dim_ftime, mean, na.rm = TRUE)
            mean.crpss[mm,sys,] <- apply(crpss.region, dim_ftime, mean, na.rm = TRUE)
            mean.enssprerr[mm,sys,] <- apply(enssprerr.region, dim_ftime, mean, na.rm = TRUE)

          } ## close if on mm
        } ## close if on sys
      } ## close if on type    
    
              
        
      #   n.vert <- n.systems
      #   int.vert <- 1:n.systems
      #   
        # if(region == 1){pos.lon <- 0:360 ; pos.lat <- 30 :-30}
        # if(region == 2){pos.lon <- 0:360 ; pos.lat <- 90 : 30}
        # if(region == 3){pos.lon <- 0:360 ; pos.lat <--30 :-90}

      # ## empty arrays with the score values to plot:        
      # mean.bias <- mean.corr <- mean.crpss <- mean.rpss <- mean.enssprerr <- array(NA, c(n.start.months, n.vert, n.ftime.months))
      # 
      # for(vert in int.vert){
      #   for(m in start.months){
      #     mm <- as.numeric(m)
      #     pos.mm <- mm + (vert - 1) * n.start.months
      #     
      #   
      #     ## select only scores inside the target region: [bias.region] = [lon, lat, n.ftime.months]
      #     bias.region <- bias[[pos.mm]][pos.lon[[region]], pos.lat[[region]], lead.months]
      #     corr.region <- corr[[pos.mm]][pos.lon[[region]], pos.lat[[region]], lead.months]
      #     rpss.region <- rpss[[pos.mm]][pos.lon[[region]], pos.lat[[region]], lead.months]
      #     crpss.region <- crpss[[pos.mm]][pos.lon[[region]], pos.lat[[region]], lead.months]
      #     enssprerr.region <- enssprerr[[pos.mm]][pos.lon[[region]], pos.lat[[region]], lead.months]
      #     
      #     ## replace values with (-)Inf with NA or the average will be (-)Inf too: (but infinite values should be absent in theory)
      #     rpss.region[which(is.infinite(rpss.region))] <- NA
      #     crpss.region[which(is.infinite(crpss.region))] <- NA
      #     
      #     ## average score values inside the target region: (note that in this case we revert back to mm instead of pos.mm)
      #     mean.bias[mm, vert,] <- apply(bias.region, 3, mean, na.rm = TRUE, drop = FALSE) ## [mean.bias] = [n.start.months, n.systems, n.ftime.months]
      #     mean.corr[mm, vert,] <- apply(corr.region, 3, mean, na.rm = TRUE, drop = FALSE)
      #     mean.rpss[mm, vert,] <- apply(rpss.region, 3, mean, na.rm = TRUE, drop = FALSE)
      #     mean.crpss[mm, vert,] <- apply(crpss.region, 3, mean, na.rm = TRUE, drop = FALSE)
      #     mean.enssprerr[mm, vert,] <- apply(enssprerr.region, 3, mean, na.rm = TRUE, drop = FALSE)
      # 
      #   } ## close if on m
      # } ## close in on vert
    #} ##close if on type 

    ## Round the scores
    mean.bias <- round(mean.bias, n.dec)
    mean.corr <- round(mean.corr, 2)
    mean.crpss <- round(mean.crpss, 2)
    mean.rpss <- round(mean.rpss, 2)
    mean.enssprerr <- round(mean.enssprerr, 2)
    
    ## For snow variables, remove the Tropics region
    shortertable <- FALSE
    if(var.name == "Snowfall" || var.name == "Snow density" || var.name == "Snow depth" || var.name == "Sea-ice cover"){
      mean.bias <- mean.bias[,-1,]
      mean.corr <- mean.corr[,-1,]
      mean.crpss <- mean.crpss[,-1,]
      mean.rpss <- mean.rpss[,-1,]
      mean.enssprerr <- mean.enssprerr[,-1,]
      region.names.figure <- region.names.figure[-1]
      n.regions <- 2
      shortertable <- TRUE
    }
    
    
    ## show target month instead of start date:
    if(scorecard_type == 'type-3' || scorecard_type == 'type-4' || scorecard_type == 'type-11' || scorecard_type == 'type-12'){ 
      ## translate values by 1 to shift from start date to target month:  
      mean.bias2 <- abind(mean.bias, mean.bias, mean.bias, along = 1)
      for(ff in 2:n.ftime.months){
        mean.bias2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.bias2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
      }
      mean.bias2 <- mean.bias2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array
      
      mean.corr2 <- abind(mean.corr, mean.corr, mean.corr, along = 1)
      for(ff in 2:n.ftime.months){
        mean.corr2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.corr2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
      }
      mean.corr2 <- mean.corr2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array
      
      mean.crpss2 <- abind(mean.crpss, mean.crpss, mean.crpss, along = 1)
      for(ff in 2:n.ftime.months){
        mean.crpss2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.crpss2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
      }
      mean.crpss2 <- mean.crpss2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array
      
      mean.rpss2 <- abind(mean.rpss, mean.rpss, mean.rpss, along = 1)
      for(ff in 2:n.ftime.months){
        mean.rpss2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.rpss2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
      }
      mean.rpss2 <- mean.rpss2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array
      
      mean.enssprerr2 <- abind(mean.enssprerr, mean.enssprerr, mean.enssprerr, along = 1)
      for(ff in 2:n.ftime.months){
        mean.enssprerr2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.enssprerr2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
      }
      mean.enssprerr2 <- mean.enssprerr2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array
    
      ## Change names to correspond with names in function
      mean.bias <- mean.bias2
      mean.corr <- mean.corr2
      mean.crpss <- mean.crpss2
      mean.rpss <- mean.rpss2
      mean.enssprerr <- mean.enssprerr2
      
      rm(mean.bias2, mean.corr2, mean.crpss2, mean.rpss2, mean.enssprerr2)
    
    }
   
    data <- list(mean.bias = mean.bias, mean.corr = mean.corr, mean.rpss = mean.rpss, mean.crpss = mean.crpss, mean.enssprerr = mean.enssprerr, 
                 region.names = region.names, region.names.figure = region.names.figure, n.regions = n.regions, 
                 n.start.months = n.start.months, n.ftime.months = n.ftime.months, n.dec = n.dec)
    
    return(data)
    
      }
