################################################################################
########## Functions for scorecard visualization for type 1 and 3 ##############
################################################################################

SC_create_table <- function(transformed_metrics){

    plevel <- "" ## TO DO:  read from recipe
    
    system.name <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_name ## 'ECMWF System 5
    reference.name <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_name ## 'ERA5'
    start.year <- recipe$Analysis$Time$hcst_start ## 1993
    end.year <- recipe$Analysis$Time$hcst_end ## 2016
    var.name <- variable_dictionary[[recipe$Analysis$Variables$name]]$long_name
    var.unit <- variable_dictionary[[recipe$Analysis$Variables$name]]$units ## 'K'
    table.column.header <- type_dictionary[[scorecard_type]]$table_column_header
    
    n.systems <- length(list('ecmwfs5', 'dwds21'))
    lead.months <- indices(recipe$Analysis$Time$ftime_min : recipe$Analysis$Time$ftime_max)
    n.ftime.months <- length(lead.months)
    
    #inverse.scale <- initialfunction_colorscale(var.name) 
    
    ## Load previously transformed metrics
    mean.bias <- transformed_metrics$mean.bias
    mean.corr <- transformed_metrics$mean.corr
    mean.rpss <- transformed_metrics$mean.rpss
    mean.crpss <- transformed_metrics$mean.crpss
    mean.enssprerr <- transformed_metrics$mean.enssprerr
    
    region.names <- transformed_metrics$region.names
    region.names.figure <- transformed_metrics$region.names.figure
    n.regions <- transformed_metrics$n.regions
    n.start.months <- transformed_metrics$n.start.months
    n.ftime.months <- transformed_metrics$n.ftime.months
    n.dec <- transformed_metrics$n.dec
    
    
    ## Define pressure levels if necessary
    if(plevel==""){
      mod.pressure <- ""
      mod.pressure.title <- ""
    }else{
      plevel <- as.integer(plevel)
      mod.pressure <- paste0("-p", plevel)
      mod.pressure.title <- paste0(" at ", plevel, " hPa")
    }

    
    ## Table row titles
    if(n.regions == 3) region.names2 <- region.names else region.names2 <- region.names[-1] 
    region.names.figure2 <- rep("", n.regions * n.ftime.months)
    
    ## replace "System" with "S" to shorten the title (so that it can fit in one row only":
   # region.names2 <- gsub("System ", "S", region.names2)
    
    if(scorecard_type == 'type-1' || scorecard_type == 'type-3'){
      for(rr in 1:n.regions) region.names.figure2[floor(n.ftime.months/2) + (rr-1) * n.ftime.months ] = region.names2[rr]
      for(rr in 1:n.regions) region.names.figure2[floor(n.ftime.months/2) + (rr-1) * n.ftime.months + 1] = region.names.figure[rr]
      forecast.month.figure <- rep(1:n.ftime.months, n.regions)
    }
    
    if(scorecard_type == 'type-2' || scorecard_type == 'type-4'){
      for(rr in 1:n.ftime.months) region.names.figure2[(rr-1) * n.regions + 1] = rr
      forecast.month.figure <- rep(region.names2, n.ftime.months)
    }
    
    if(scorecard_type == 'type-9' || scorecard_type == 'type-11'){
      region.names.figure2 <- rep("", n.systems * n.ftime.months)
      system.name2 <- gsub("System ", "S", system.name)
      for(ss in 1:n.systems) region.names.figure2[floor(n.ftime.months/2) + (ss-1) * n.ftime.months ] = system.name2[ss]
      forecast.month.figure <- rep(1:n.ftime.months, n.systems)
    }
    
    if(scorecard_type == 'type-10' ||  scorecard_type == 'type-12'){
      region.names.figure2 <- rep("", n.systems * n.ftime.months)
      system.name2 <- gsub("System ", "S", system.name)
      for(ss in 1:n.ftime.months) region.names.figure2[ceiling(n.systems/2) + (ss-1) * n.systems ] = ss
      forecast.month.figure <- rep(system.name2, n.ftime.months)
    }    
    
    #if(scorecard_type == 'type-9' || scorecard_type == 'type-10' || scorecard_type == 'type-11' || scorecard_type == 'type-12') n.regions <- n.systems
    
    ## prepare scores to insert them in the data.frame:
    bias.matrix <- corr.matrix <- crpss.matrix <- rpss.matrix <- enssprerr.matrix <- array(NA, c(n.regions * n.ftime.months, n.start.months))
    
    for(col in 1:n.start.months){
      ## [mean.bias] = [n.start.months, n.systems, n.ftime.months]
      bias.matrix[, col] <- as.vector(t(mean.bias[col,,]))
      corr.matrix[, col] <- as.vector(t(mean.corr[col,,]))
      crpss.matrix[, col] <- as.vector(t(mean.crpss[col,,]))
      rpss.matrix[, col] <- as.vector(t(mean.rpss[col,,]))
      enssprerr.matrix[, col] <- as.vector(t(mean.enssprerr[col,,]))
    }
    
    ## add column names:
    colnames(bias.matrix) <- paste0("BiasTarget", 1:n.start.months) ### Necessary ??
    colnames(corr.matrix) <- paste0("CorrTarget", 1:n.start.months)
    colnames(crpss.matrix) <- paste0("CrpssTarget", 1:n.start.months)
    colnames(rpss.matrix) <- paste0("RpssTarget", 1:n.start.months)
    colnames(enssprerr.matrix) <- paste0("EnsSprErrTarget", 1:n.start.months)
    
    ## swap row order if forecast month is shown in column 1 before the region:
    if(scorecard_type == 'type-2' || scorecard_type == 'type-4' || scorecard_type == 'type-10' || scorecard_type == 'type-12'){
      bias.matrix2 <- corr.matrix2 <- crpss.matrix2 <- rpss.matrix2 <- enssprerr.matrix2 <- array(NA, c(n.regions * n.ftime.months, n.start.months))
      
      for(rrow in 1:(n.ftime.months * n.regions)){
        rrow2 <- 1 + ((rrow-1) %% n.regions) * n.ftime.months + floor((rrow-1)/n.regions)
        bias.matrix2[rrow,] <- bias.matrix[rrow2,]
        corr.matrix2[rrow,] <- corr.matrix[rrow2,]
        crpss.matrix2[rrow,] <- crpss.matrix[rrow2,]
        rpss.matrix2[rrow,] <- rpss.matrix[rrow2,]
        enssprerr.matrix2[rrow,] <- enssprerr.matrix[rrow2,]
      }    
      
      bias.matrix <- bias.matrix2
      corr.matrix <- corr.matrix2
      crpss.matrix <- crpss.matrix2
      rpss.matrix <- rpss.matrix2
      enssprerr.matrix <- enssprerr.matrix2
      
      rm(bias.matrix2, corr.matrix2, crpss.matrix2, rpss.matrix2, enssprerr.matrix2)
    }
    
    ## in case of a single region we need to remove the first column:
    if(length(region.names) == 1 && scorecard_type == 'type-1' || scorecard_type == 'type-2' || scorecard_type == 'type-3' || scorecard_type == 'type-4'
       ) mod.single <- 1 else mod.single <- 0

    ## create data.frame with all values to show in the scorecard:
    if(mod.single == 0) table.text <- data.frame(Region = region.names.figure2, Forecast_month = forecast.month.figure)
    if(mod.single == 1 && scorecard_type == 'type-1' || scorecard_type == 'type-2' || scorecard_type == 'type-3' || scorecard_type == 'type-4')
      table.text <- data.frame(Forecast_month = forecast.month.figure)
    if(mod.single == 1 && scorecard_type == 'type-9' || scorecard_type == 'type-10' || scorecard_type == 'type-11' || scorecard_type == 'type-12')
      table.text <- data.frame(Region = region.names.figure2, Forecast_month = forecast.month.figure)

    table <- cbind(table.text, bias.matrix, corr.matrix, crpss.matrix, rpss.matrix, enssprerr.matrix)
    
    # Cell colors of Bias metric

    bias.max <- max(abs(mean.bias), na.rm = TRUE)
    bias.minmax_max <- quantile(mean.bias, 0.98, na.rm = TRUE)
    bias.minmax_min <- quantile(mean.bias, 0.02, na.rm = TRUE)
    bias.max <- max(abs(bias.minmax_min), abs(bias.minmax_max))
    bias.int <- bias.max * c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1) ## one point more than the colors below (the intervals)

    bias.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
    colorunder_bias <- "#04040E"
    colorsup_bias <- "#730C04"

    bias.col <- array(colorunder_bias, c(n.ftime.months * n.regions, n.start.months)) ## gray is to detect outliers
    bias.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.start.months)) ## black is the default text color
    bias.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.start.months)) ## TRUE is the default bold value

    for(mm in 1:n.start.months){
      for(ff in 1:n.ftime.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(bias.int)-1)){
            if(is.nan(mean.bias[mm, rr, ff])){ ##
              bias.col[ff + (rr-1) * n.ftime.months, mm] <- "gray"
            } else {
              if(bias.matrix[ff + (rr-1) * n.ftime.months, mm] >= bias.int[pp] && bias.matrix[ff + (rr-1) * n.ftime.months, mm] <= bias.int[pp+1])
                bias.col[ff + (rr-1) * n.ftime.months, mm] <- bias.palette[pp]
              if(bias.matrix[ff + (rr-1) * n.ftime.months, mm] < bias.int[1])
                 bias.col[ff + (rr-1) * n.ftime.months, mm] <- colorunder_bias
              if(bias.matrix[ff + (rr-1) * n.ftime.months, mm] >= bias.int[length(bias.int)])
                bias.col[ff + (rr-1) * n.ftime.months, mm] <- colorsup_bias          
            }
            
    
            ## color text in white and bold if background is white or dark blue or dark red:
            if(is.nan(bias.matrix[ff + (rr-1) * n.ftime.months, mm]) ||
               (!is.nan(bias.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 1 && bias.matrix[ff + (rr-1) * n.ftime.months, mm] < bias.int[2]) ||
               (!is.nan(bias.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 2 && bias.matrix[ff + (rr-1) * n.ftime.months, mm] < bias.int[3]) ||
               (!is.nan(bias.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(bias.int) - 1) && bias.matrix[ff + (rr-1) * n.ftime.months, mm] >= bias.int[length(bias.int) - 1]) ||
               (!is.nan(bias.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(bias.int) - 2) && bias.matrix[ff + (rr-1) * n.ftime.months, mm] >= bias.int[length(bias.int) - 2]))
             
             {
              bias.text.col[ff + (rr-1) * n.ftime.months, mm] <- "white"
              bias.text.bold[ff + (rr-1) * n.ftime.months, mm] <- TRUE
             }
          }
        }
      }
    }

    ## round the intervals to show them in the legend bar:
    bias.int <- round(bias.int, n.dec)
    
    ## cell colors of Correlation metric:
    Corr.int <- c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1) ## one point more than the colors below (the intervals)
    Corr.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
    
    Corr.col <- array("gray", c(n.ftime.months * n.regions, n.start.months)) ## gray is to detect outliers
    Corr.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.start.months)) ## black is the default text color
    Corr.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.ftime.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(Corr.int)-1)){
            if(is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm])){
              Corr.col[ff + (rr-1) * n.ftime.months, mm] <- "gray"
            } else {
              if(corr.matrix[ff + (rr-1) * n.ftime.months, mm] >= Corr.int[pp] && corr.matrix[ff + (rr-1) * n.ftime.months, mm] <= Corr.int[pp+1])
                Corr.col[ff + (rr-1) * n.ftime.months, mm] <- Corr.palette[pp]
            }
            
            ## color text in white and bold if background is white or dark blue or dark red:                    
            if(is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm]) ||
               (!is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 1 && corr.matrix[ff + (rr-1) * n.ftime.months, mm] < Corr.int[2]) ||
               (!is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 2 && corr.matrix[ff + (rr-1) * n.ftime.months, mm] < Corr.int[3]) ||
               (!is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(Corr.int) - 1) && corr.matrix[ff + (rr-1) * n.ftime.months, mm] >= Corr.int[length(Corr.int) - 1]) ||
               (!is.nan(corr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(Corr.int) - 2) && corr.matrix[ff + (rr-1) * n.ftime.months, mm] >= Corr.int[length(Corr.int) - 2]))
            {
              Corr.text.col[ff + (rr-1) * n.ftime.months, mm] <- "white"
              Corr.text.bold[ff + (rr-1) * n.ftime.months, mm] <- TRUE
            }
          }
        }
      }
    }
    
    ## cell colors of CRPSS metric:
    CRPSS.int <- c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5) ## one point more than the colors below (the intervals)
    CRPSS.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
    colorunder <- "#04040E" ## for both CRPSS and RPSS
    colorsup <- "#730C04" ## for both CRPSS and RPSS
    
    CRPSS.col <- array(colorunder, c(n.ftime.months * n.regions, n.start.months)) ## blue is to detect outliers
    CRPSS.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.start.months)) ## black is the default text color
    CRPSS.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.ftime.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(CRPSS.int)-1)){
            if(is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm])){
              CRPSS.col[ff + (rr-1) * n.ftime.months, mm] <- "gray"
            } else {
              if(crpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= CRPSS.int[pp] && crpss.matrix[ff + (rr-1) * n.ftime.months, mm] <= CRPSS.int[pp+1])
                CRPSS.col[ff + (rr-1) * n.ftime.months, mm] <- CRPSS.palette[pp]
              if(crpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= CRPSS.int[length(CRPSS.int)-1])
                CRPSS.col[ff + (rr-1) * n.ftime.months, mm] <- colorsup
              if(crpss.matrix[ff + (rr-1) * n.ftime.months, mm] < CRPSS.int[1])
                CRPSS.col[ff + (rr-1) * n.ftime.months, mm] <- colorunder
            }  
              
            ## color text in white and bold if background is white or dark blue or dark red:                    
            if(is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm]) ||
               (!is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 1 && crpss.matrix[ff + (rr-1) * n.ftime.months, mm] < CRPSS.int[2]) ||
               (!is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 2 && crpss.matrix[ff + (rr-1) * n.ftime.months, mm] < CRPSS.int[3]) ||
               (!is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(CRPSS.int) - 1) && crpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= CRPSS.int[length(CRPSS.int) - 1]) ||
               (!is.nan(crpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(CRPSS.int) - 2) && crpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= CRPSS.int[length(CRPSS.int) - 2]))
            {
              CRPSS.text.col[ff + (rr-1) * n.ftime.months, mm] <- "white"
              CRPSS.text.bold[ff + (rr-1) * n.ftime.months, mm] <- TRUE
            }
          }
        }
      }
    }
    

## cell colors of RPSS metric:

RPSS.int <- CRPSS.int ## c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5) ## one point more than the colors below (the intervals)
RPSS.palette <- CRPSS.palette ## c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')

RPSS.col <- array(colorunder, c(n.ftime.months * n.regions, n.start.months)) ## "#045a8d" blue is to detect outliers
RPSS.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.start.months)) ## black is the default text color
RPSS.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.start.months)) ## TRUE is the default bold value

for(mm in 1:n.start.months){
  for(ff in 1:n.ftime.months){
    for(rr in 1:n.regions){
      for(pp in 1:(length(RPSS.int)-1)){
        if(is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm])){
          RPSS.col[ff + (rr-1) * n.ftime.months, mm] <- "gray"
        } else {
          if(rpss.matrix[ff + (rr-1) * n.ftime.months, mm] < RPSS.int[1])
            RPSS.col[ff + (rr-1) * n.ftime.months, mm] <- colorunder
          if(rpss.matrix[ff + (rr-1) * n.ftime.months, mm] > RPSS.int[length(RPSS.int)])
            RPSS.col[ff + (rr-1) * n.ftime.months, mm] <- colorsup
         if(rpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= RPSS.int[pp] && rpss.matrix[ff + (rr-1) * n.ftime.months, mm] <= RPSS.int[pp+1])
            RPSS.col[ff + (rr-1) * n.ftime.months, mm] <- RPSS.palette[pp]
          
        }
 
        ## color text in white if background is white or deep blue:
        if(is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm]) ||
           (!is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 1 && rpss.matrix[ff + (rr-1) * n.ftime.months, mm] < RPSS.int[2]) ||
           (!is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 2 && rpss.matrix[ff + (rr-1) * n.ftime.months, mm] < RPSS.int[3]) ||
           (!is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(RPSS.int) - 1) && rpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= RPSS.int[length(RPSS.int) - 1]) ||
           (!is.nan(rpss.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(RPSS.int) - 2) && rpss.matrix[ff + (rr-1) * n.ftime.months, mm] >= RPSS.int[length(RPSS.int) - 2]))
        {
          RPSS.text.col[ff + (rr-1) * n.ftime.months, mm] <- "white"
          RPSS.text.bold[ff + (rr-1) * n.ftime.months, mm] <- TRUE
        }
      }
    }
  }
}
    
    ## cell colors of EnsSprErr metric
    EnsSprErr.int <- c(0, 0.6, 0.7, 0.8, 0.9, 1, 1.2, 1.4, 1.6, 1.8, 2)
    EnsSprErr.palette <- CRPSS.palette
    
    EnsSprErr.col <- array(colorunder, c(n.ftime.months * n.regions, n.start.months)) ## "#045a8d" blue is to detect outliers
    EnsSprErr.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.start.months)) ## black is the default text color
    EnsSprErr.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.start.months)) ## TRUE is the default bold value

    
    for(mm in 1:n.start.months){
      for(ff in 1:n.ftime.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(EnsSprErr.int)-1)){
            if(is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm])){
              EnsSprErr.col[ff + (rr-1) * n.ftime.months, mm] <- "gray"
            } else {
              if(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] < EnsSprErr.int[1])
                EnsSprErr.col[ff + (rr-1) * n.ftime.months, mm] <- colorunder
              if(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] > EnsSprErr.int[length(EnsSprErr.int)-1])
                EnsSprErr.col[ff + (rr-1) * n.ftime.months, mm] <- colorsup
              if(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] >= EnsSprErr.int[pp] && enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] <= EnsSprErr.int[pp+1])
                EnsSprErr.col[ff + (rr-1) * n.ftime.months, mm] <- EnsSprErr.palette[pp]
            }
            ## color text in white if background is white or deep blue:
            if(is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm]) ||
               (!is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 1 && enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] < EnsSprErr.int[2]) ||
               (!is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == 2 && enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] < EnsSprErr.int[3]) ||
               (!is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(EnsSprErr.int) - 1) && enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] >= EnsSprErr.int[length(EnsSprErr.int) - 1]) ||
               (!is.nan(enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm]) && pp == (length(EnsSprErr.int) - 2) && enssprerr.matrix[ff + (rr-1) * n.ftime.months, mm] >= EnsSprErr.int[length(EnsSprErr.int) - 2]))
            {
              EnsSprErr.text.col[ff + (rr-1) * n.ftime.months, mm] <- "white"
              EnsSprErr.text.bold[ff + (rr-1) * n.ftime.months, mm] <- TRUE
            }
          }
        }
      }
    }

    ## table header:
    n.metrics <- 5 ### TO DO: set as variable from recipe
    n.columns <- 2 + n.start.months * n.metrics
    
    if(scorecard_type == 'type-1' || scorecard_type == 'type-3' ) my.col.names <- c("Region", "Forecast month", rep(c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"), n.metrics))
    if(scorecard_type == 'type-2' || scorecard_type == 'type-4' ) my.col.names <- c("Forecast month", "Region", rep(c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"), n.metrics))
    if(scorecard_type == 'type-9' || scorecard_type == 'type-11') my.col.names <- c("System", "Forecast month", rep(c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"), n.metrics))
    if(scorecard_type == 'type-10'|| scorecard_type == 'type-12') my.col.names <- c("Forecast month", "System", rep(c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"), n.metrics))

    ## Add capital to first letter and remove the capitals in the second part of the variable name:
    var.name.min <- paste0(toupper(substring(var.name, 1, 1)), substring(str_to_lower(var.name), 2, nchar(var.name)))
    
    options(stringsAsFactors = FALSE)
    my.header.names <- as.data.frame(data.frame(c1=c("", paste0("Mean Bias (", var.unit,")"), "Correlation", "CRPSS", "RPSS", "Spread-to-Error Ratio"), c2=c(2, rep(n.start.months, n.metrics))))
    
    if(mod.single == 0) mod.single.title <- "" else mod.single.title <- paste0(" for ", region.names)
    my.header.names2 <- data.frame(c1=paste0(var.name.min, mod.pressure.title, " of ", system.name, mod.single.title), c2 = n.columns)
    
    if(scorecard_type == 'type-9' || scorecard_type == 'type-10' || scorecard_type == 'type-11' || scorecard_type == 'type-12'){
      my.header.names2 <- data.frame(c1=paste0(var.name.min, mod.level.title, ". Region: ", region.names[region], " ", region.names.figure[region]), c2=n.columns)
    }
    
    ref.label <- data.frame(c1=paste0("(Ref: ", reference.name, " ", start.year, "-", end.year, ")"), c2=n.columns) 
    column_title <- as.data.frame(data.frame(c1=c("", paste0(rep(table.column.header, n.metrics))), c2=c(2, rep(n.start.months, n.metrics))))
    
    
    ## as above, but without normalization:
    table2 <- table
    
    ## Select the number of decimals to show, their font size and color
    for(col in 3:n.columns) {
      if (col > (2 + n.start.months)) table2[,col] <- format(table[,col], nsmall = 2) else table2[,col] <- format(table[,col], nsmall = n.dec) 
      table2[,col] <- cell_spec(table2[,col], font_size = 8, color="gray70") ## smaller font size
    }
    
    ## Replace with "No Data" cells with NaN
    for(i in 3:n.columns) table2[,i] <- str_replace(table2[,i], "NaN", "")
    
    ## Change the font size of any value in the table lower than -1000 with a smaller one, to fit in its cell: (this usually happens with CRPSS)
    for(i in 3:n.columns) { ss <- which(table[,i] < -1000) ; if(length(ss) > 0) table2[ss,i] <- str_replace(table2[ss,i], "8px", "5px")}


    ##### Create html table  #####
    table.html.part <- list()
    table.html.part[[1]] <- kbl(table2, escape = F, col.names = my.col.names, align = rep("c", n.columns)) %>%
    #table.html <- kbl(table2, escape = F, col.names=my.col.names, align = rep("c", n.columns)) %>%
      kable_paper("hover", full_width = F) %>%
      add_header_above(header = column_title, font = 14) %>%
      add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
      add_header_above(header = my.header.names, font_size = 20) %>%
      add_header_above(header = my.header.names2, font_size = 20, align = "left")

    for(index in 1:n.metrics){ ## score
      if(index == 1) { my.background <- bias.col; my.text.color <- bias.text.col; my.bold <- bias.text.bold}
      if(index == 2) { my.background <- Corr.col; my.text.color <- Corr.text.col; my.bold <- Corr.text.bold}
      if(index == 3) { my.background <- CRPSS.col; my.text.color <- CRPSS.text.col; my.bold <- CRPSS.text.bold}
      if(index == 4) { my.background <- RPSS.col; my.text.color <- RPSS.text.col; my.bold <- RPSS.text.bold}
      if(index == 5) { my.background <- EnsSprErr.col; my.text.color <- EnsSprErr.text.col; my.bold <- EnsSprErr.text.bold}
      
      for(stm in 1:n.start.months){ ## loop on start date             
        table.html.part[[(index - 1) * n.start.months + stm + 1]] <-
          column_spec(table.html.part[[(index - 1) * n.start.months + stm]], 2 + n.start.months * (index - 1) + stm, background = my.background[,stm], color = my.text.color[,stm], bold = my.bold[,stm])
      }
    }
    
    n.last.list <- n.metrics * n.start.months + 1

   if(scorecard_type == 'type-1' || scorecard_type == 'type-3'|| scorecard_type == 'type-9' || scorecard_type == 'type-11'){
      table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
        column_spec(1, border_left = "4px solid black") %>%
        column_spec(2, bold = TRUE, width_max = "1cm") %>%
        column_spec(3:n.columns, width_max = "0.9cm", width_min = "0.9cm") %>% 
        column_spec(c(1, 2, 2  + n.start.months, 2  + n.start.months*2, 2 + n.start.months*3, 2 + n.start.months*4), border_right = "1px solid black") %>%
        column_spec(n.columns, border_right = "4px solid black") %>%
        row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.ftime.months), extra_css = "border-bottom: 2px solid black", hline_after = TRUE)
    }

    if(scorecard_type == 'type-2' || scorecard_type == 'type-4'|| scorecard_type == 'type-10' || scorecard_type == 'type-12'){
      table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "1cm", width_max = "1cm") %>%
        column_spec(1, border_left = "4px solid gray") %>% ## why gray not black?? 
        column_spec(2 , bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
        column_spec(3:n.columns, width_max = "0.9cm", width_min = "0.9cm") %>%
        column_spec(c(1, 2 , 2  + n.start.months, 2  + n.start.months*2, 2  + n.start.months*3,  2 + n.start.months*4), border_right = "1px solid black") %>%
        column_spec(n.columns, border_right = "4px solid gray") %>% ## why gray not black?? 
        row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.regions), extra_css = "border-bottom: 2px solid black", hline_after = TRUE)
    }

    
      ## Create colour bar legends
      png(filename= paste0(fileout, "/scorecard_legend.png"), width=455, height=40)
      ColorBar(brks = CRPSS.int, cols = CRPSS.palette, vertical = FALSE, var_limits=c(-10000000, 1), col_inf = colorunder, col_sup = colorsup) # "#000080")
      dev.off()
      
      png(filename= paste0(fileout, "/scorecard_legend_bias.png"), width=455, height=40)
      ColorBar(brks = round(bias.int, n.dec), cols = bias.palette, vertical = FALSE, triangle_ends_scale = 1, col_inf = colorunder_bias, col_sup = colorsup_bias) 
      dev.off()
      
      png(filename= paste0(fileout, "/scorecard_legend_corr.png"), width=455, height=40)
      ## Paste all legends together
      system(paste0("montage ./scorecard_legend_bias_full.png ./scorecard_legend_corr.png ./scorecard_legend.png ./scorecard_legend.png ./scorecard_legend_enssprerr.png -tile 5x1 -geometry -4+10 ./legend_scorecard.png"))
      unlink('scorecard_legend*')
      setwd(currentworkingdir)
      
      ## Save html table as png
      my.file <- paste0(fileout, '/scorecard.png')
      try(save_kable(table.html, file = my.file), silent = TRUE)
      
      ## Remove temporal directories (not automatically removed due to error with save_kable)
      unlink(paste0(fileout, './saved_results/lib*'), recursive = TRUE) # clean temporal directories (not automatically removed because of error with save_kable)
      
      ## Add white space to the top of the scorecards, to be cut when adding legend:
      setwd(fileout)
      system(paste0("convert ./scorecard.png -background white -splice 0x20 ./scorecard.png"))
      
      ## Add the legends below the scorecard table
      system(paste0("montage ./scorecard.png ./legend_scorecard.png -tile 1x2 -geometry +0-10 ./scorecard.png"))
      
}
